﻿/*
 * CKEditor - The text editor for Internet - http://ckeditor.com
 * Copyright (C) 2003-2008 Frederico Caldeira Knabben
 *
 * == BEGIN LICENSE ==
 *
 * Licensed under the terms of any of the following licenses at your
 * choice:
 *
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 *
 *  - GNU Lesser General Public License Version 2.1 or later (the "LGPL")
 *    http://www.gnu.org/licenses/lgpl.html
 *
 *  - Mozilla Public License Version 1.1 or later (the "MPL")
 *    http://www.mozilla.org/MPL/MPL-1.1.html
 *
 * == END LICENSE ==
 */

/**
 * Ajax methods for data loading.
 * @namespace
 * @example
 */
CKEDITOR.ajax = (function()
{
	var createXMLHttpRequest = function()
	{
		// In IE, using the native XMLHttpRequest for local files may throw
		// "Access is Denied" errors.
		if ( !CKEDITOR.env.IE || location.protocol != 'file:' )
			try { return new XMLHttpRequest(); } catch(e) {}

		try { return new ActiveXObject( 'Msxml2.XMLHTTP' ); } catch (e) {}
		try { return new ActiveXObject( 'Microsoft.XMLHTTP' ); } catch (e) {}

		return null;
	};

	var checkStatus = function( xhr )
	{
		// HTTP Status Codes:
		//	 2xx : Success
		//	 304 : Not Modified
		//	   0 : Returned when running locally (file://)
		//	1223 : IE may change 204 to 1223 (see http://dev.jquery.com/ticket/1450)

		return ( xhr.readyState == 4 &&
				(	( xhr.status >= 200 && xhr.status < 300 ) ||
					xhr.status == 304 ||
					xhr.status === 0 ||
					xhr.status == 1223 ) );
	};

	var getResponseText = function( xhr )
	{
		if ( checkStatus( xhr ) )
			return xhr.responseText;
		return null;
	};

	var getResponseXml = function( xhr )
	{
		if ( checkStatus( xhr ) )
		{
			var xml = xhr.responseXML;
			return new CKEDITOR.xml( xml && xml.firstChild ? xml : xhr.responseText );
		}
		return null;
	};

	var load = function( url, callback, getResponseFn )
	{
		var async = !!callback;

		var xhr = createXMLHttpRequest();

		if ( !xhr )
			return null;

		xhr.open( 'GET', url, async );

		if ( async )
		{
			// TODO: perform leak checks on this closure.
			/** @ignore */
			xhr.onreadystatechange = function()
			{
				if ( xhr.readyState == 4 )
				{
					callback( getResponseFn( xhr ) );
					xhr = null;
				}
			};
		}

		xhr.send(null);

		return async ? '' : getResponseFn( xhr );
	};

	return 	/** @lends CKEDITOR.ajax */ {

		/**
		 * Loads data from an URL as plain text.
		 * @param {String} url The URL from which load data.
		 * @param {Function} [callback] A callback function to be called on
		 *		data load. If not provided, the data will be loaded
		 *		asynchronously, passing the data value the function on load.
		 * @returns {String} The loaded data. For asynchronous requests, an
		 *		empty string. For invalid requests, null.
		 * @example
		 * // Load data synchronously.
		 * var data = CKEDITOR.ajax.load( 'somedata.txt' );
		 * alert( data );
		 * @example
		 * // Load data asynchronously.
		 * var data = CKEDITOR.ajax.load( 'somedata.txt', function( data )
		 *     {
		 *         alert( data );
		 *     } );
		 */
		load : function( url, callback )
		{
			return load( url, callback, getResponseText );
		},

		/**
		 * Loads data from an URL as XML.
		 * @param {String} url The URL from which load data.
		 * @param {Function} [callback] A callback function to be called on
		 *		data load. If not provided, the data will be loaded
		 *		asynchronously, passing the data value the function on load.
		 * @returns {CKEDITOR.xml} An XML object holding the loaded data. For asynchronous requests, an
		 *		empty string. For invalid requests, null.
		 * @example
		 * // Load XML synchronously.
		 * var xml = CKEDITOR.ajax.loadXml( 'somedata.xml' );
		 * alert( xml.getOuterXml() );
		 * @example
		 * // Load XML asynchronously.
		 * var data = CKEDITOR.ajax.loadXml( 'somedata.xml', function( xml )
		 *     {
		 *         alert( xml.getOuterXml() );
		 *     } );
		 */
		loadXml : function( url, callback )
		{
			return load( url, callback, getResponseXml );
		}
	};
})();
